package com.nisum.spark.consumer;

import com.nisum.spark.dto.Product;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.types.StructType;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import static com.nisum.spark.util.Constants.*;

public class Consumer {

    public static void main(String[] args) throws TimeoutException {
        SparkSession sparkSession = SparkSession
                .builder().master("local[*]").appName("Spark Streaming Example 2").getOrCreate();

        Map<String, String> kafkaConfigMap = new HashMap<>();
        kafkaConfigMap.put("kafka.bootstrap.servers", HOST);
        kafkaConfigMap.put("subscribe", TOPIC);
        kafkaConfigMap.put("startingOffsets", "earliest");
        kafkaConfigMap.put("failOnDataLoss", "false");
        kafkaConfigMap.put("maxOffsetsPerTrigger", "100");

        Dataset<Row> df = sparkSession
                .readStream()
                .format("kafka")
                .options(kafkaConfigMap)
                .load();

        if (df != null) {
            StructType structType = Encoders.bean(Product.class).schema();

            Dataset<Row> df1 = df
                    .selectExpr("CAST(value as String) as message")
                    .select(functions.from_json(functions.col("message"), structType).as("data"))
                    .select("data.*");

            Dataset<Row> filteredDataFrame = df1.filter(functions.col("deliveryStatus").equalTo("shipped"))
                    .filter(functions.col("date").equalTo("25-05-2023"));

            Dataset<Product> ds = filteredDataFrame.as(Encoders.bean(Product.class));

            StreamingQuery streamingQuery = writeStreamingData(ds);
            try {
                streamingQuery.awaitTermination();
            } catch (Exception e) {
                e.printStackTrace();
                streamingQuery.stop();
                sparkSession.stop();
            }
        }

    }

    private static StreamingQuery writeStreamingData(Dataset<Product> ds) throws TimeoutException {
        return ds
                .writeStream()
                .outputMode("append")
                .format("csv")
                //.option("checkpointLocation","checkpointLocation/jobs")
                .trigger(org.apache.spark.sql.streaming.Trigger.ProcessingTime("10 seconds"))
                .foreachBatch((batchDF, batchId) -> {
                    batchDF.write().format("csv").mode("append").save("src/main/resources/productDataOutPutCSV");
                })
                .start();
    }
}
